/**********************************************************************************************************************
 * @file            CCA.h
 *
 * @brief           The Cloud Communication Application (CCA) handles all communication via MQTT with the Cloud (AWS). 
 *
 * @author          Devan Hone
 * @date            06/07/2024
 * @copyright       2024 Lion Energy LLC.
 *                  All rights reserved
 *
 *********************************************************************************************************************/
/**
 * @addtogroup      CCA
 *                  Cloud Communication Application Module
 *
 * @ref             CCA.h
 *
 *********************************************************************************************************************/

#include "COD_Compiler_Defines.h"

#ifndef LION_CCA_H__
#define LION_CCA_H__

#include "lion/icc/ICC.h"
#include "lion/rtos/RTOS_Timeout.h"

#ifdef __cplusplus
extern "C"
{
#endif

/**********************************************************************************************************************
* @cond                                              GLOBAL SCOPE                                             @endcond
*********************************************************************************************************************/

/**
 * @brief Initialize the CCA module. This should only be called once.
 * @warning No other functions in the module should be called until CCA_Init is called.
 */
GlobScop void CCA_Init(void);

/**
 * @brief Starts the CCA module
 *
 * @returns 0 on success, -EALREADY if already running.
*/
GlobScop int CCA_Start();

/**
 * @brief
 * @param tim_Timeout Timeout to wait for module to shutdown, or RTOS_NO_WAIT to return immediately regardless of success.
 *
 * @returns 0 on success or if tim_Timeout is RTOS_NO_WAIT
 * @returns -EAGAIN if timed out waiting for module to shutdown
 */
GlobScop int CCA_Stop(RTOS_Timeout_t tim_Timeout);

/**
 * @brief Attempts provisioning with Cloud Provider (wraps cca_aws_RegisterThing())
 *
 * @returns EXIT_SUCCESS or 0 upon success, -EXIT_FAILURE upon failure
*/
GlobScop int CCA_Provision();

#ifdef __cplusplus
}
#endif

#endif /* LION_CCA_H__ */
//EOF
