#include "include/CCA_Dcdc.h"
#include "include/CCA_Internal.h"
#include "include/CCA_IccCalls.h"
#include "lion/log/log.h"


#define CCA_DCDC_PARAMS_SIZE sizeof(SCH_DcdcParameters_t)


LOG_MODULE_REGISTER(CCA_dcdc, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

FileScop SCH_DcdcParameters_t str_DcdcParameters;

FileScop const char *arr_DcdcSettingKeys[] = {
    #define X_DCDC(param) #param,
    DCDC_SETTINGS_XMACRO
    DCDC_DEVICE_INFO_XMACRO
    #undef X_DCDC
};
FileScop const CCA_KeyToSchemaParamMap_t arr_str_DcdcSettings[] = {   
    #define X_DCDC(param) {#param, (int32_t*)&str_DcdcParameters.settings.param, NULL},
    DCDC_SETTINGS_XMACRO
    #undef X_DCDC
    #define X_DCDC(param) {#param, (int32_t*)&str_DcdcParameters.deviceInfo.param, NULL},
    DCDC_DEVICE_INFO_XMACRO
    #undef X_DCDC
};
FileScop const size_t siz_NUM_DCDC_SETTING_KEYS = sizeof(arr_str_DcdcSettings) / sizeof(arr_str_DcdcSettings[0]);

FileScop const CCA_KeyToSchemaParamMap_t arr_str_DcdcProdData[] = {
    {"pv_w", (int32_t *)&str_DcdcParameters.componentData.pv_w, NULL},
    {"pv_kwh_total", NULL, (uint32_t *)&str_DcdcParameters.historicalData.pv_kwh_total}
};
FileScop const size_t siz_NUM_PROD_KEYS = sizeof(arr_str_DcdcProdData) / sizeof(arr_str_DcdcProdData[0]);

FileScop const CCA_KeyToSchemaParamMap_t arr_str_DcdcCompData[] = {
    #define X_DCDC(parameter) {#parameter, (int32_t*)&str_DcdcParameters.componentData.parameter, NULL},
    DCDC_COMPONENT_DATA_XMACRO
    #undef X_DCDC
    #define X_DCDC(parameter) {#parameter, NULL, (uint32_t*)&str_DcdcParameters.historicalData.parameter},
    DCDC_HISTORICAL_DATA_XMACRO
    #undef X_DCDC
};
FileScop const size_t siz_NUM_COMP_KEYS = sizeof(arr_str_DcdcCompData) / sizeof(arr_str_DcdcCompData[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_DcdcAlarmCodes[] = {
    #define X_DCDC(code) {#code, offsetof(SCH_DcdcCodes_t, alarm.code)},
    DCDC_CODES_ALARM_XMACRO
    #undef X_DCDC
};
FileScop const size_t siz_NUM_ALARM_KEYS = sizeof(arr_str_DcdcAlarmCodes) / sizeof(arr_str_DcdcAlarmCodes[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_DcdcFaultCodes[] = {
    #define X_DCDC(code) {#code, offsetof(SCH_DcdcCodes_t, fault.code)},
    DCDC_CODES_FAULT_XMACRO
    #undef X_DCDC
};
FileScop const size_t siz_NUM_FAULT_KEYS = sizeof(arr_str_DcdcFaultCodes) / sizeof(arr_str_DcdcFaultCodes[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_DcdcStatusCodes[] = {
    #define X_DCDC(code) {#code, offsetof(SCH_DcdcCodes_t, status.code)},
    DCDC_CODES_STATUS_XMACRO
    #undef X_DCDC
};
FileScop const size_t siz_NUM_STATUS_KEYS = sizeof(arr_str_DcdcStatusCodes) / sizeof(arr_str_DcdcStatusCodes[0]);

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

ModScop void CCA_Dcdc_Init(void)
{
    memset(&str_DcdcParameters, 0, CCA_DCDC_PARAMS_SIZE);
}

ModScop int CCA_Dcdc_InitShadows(void)
{
    bool bol_CommFailure = false;
    int int_rc = EXIT_SUCCESS;

    // Subscribe to DCDC_Settings Shadow sub-topics
    int_rc = CCA_AWS_SubscribeToShadowSubtopics(CCA_sz_DCDC_SHADOW_NAME, cca_sz_dcdc_update_delta_topic, 
                                                cca_sz_dcdc_update_accepted_topic, cca_sz_dcdc_update_rejected_topic);
    // Publish DCDC Settings
    if (int_rc == EXIT_SUCCESS)
    {
        ArduinoJson::StaticJsonDocument<CCA_u16_SETTINGS_JSON_DOC_SIZE> jd_DcdcSettingsReported;
        ArduinoJson::JsonObject jo_Product = jd_DcdcSettingsReported["state"]["reported"].createNestedObject("product");
        jo_Product["id"] = cca_sz_thing_name;
        jo_Product["name"] = cca_sz_product_name;
        ArduinoJson::JsonObject jo_ProductDatum = jo_Product.createNestedObject("datum");
        
        
        uint32_t u32_DcdcParametersSize = CCA_DCDC_PARAMS_SIZE;
        int_rc = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_DCDC, 0, &str_DcdcParameters, &u32_DcdcParametersSize);
        if (int_rc != EXIT_SUCCESS || u32_DcdcParametersSize != CCA_DCDC_PARAMS_SIZE)
        {
            LOG_INF("Failed to getEMSData for %s, index: 0, bytes: %d, with Error: %i", 
                        CCA_sz_UNIT_TYPE_DCDC, u32_DcdcParametersSize, int_rc);
            bol_CommFailure = true;
        }
        
        //Populate object datum and copy data into new array for synchronizing the DCDCs
        char* ptr_sz_SyncBuffer[siz_NUM_DCDC_SETTING_KEYS];
        char arr_sz_SyncBuffer[siz_NUM_DCDC_SETTING_KEYS][MAX_VALUE_SIZE];
        for (uint8_t i = 0; i < siz_NUM_DCDC_SETTING_KEYS; i++)
        {
            char sz_TempValue[MAX_VALUE_SIZE];
            if (arr_str_DcdcSettings[i].ptr_s32_Val != NULL && 
                *arr_str_DcdcSettings[i].ptr_s32_Val != COD_s32_INVALID_DATA)
            {
                sprintf(sz_TempValue, "%d", *arr_str_DcdcSettings[i].ptr_s32_Val);
            }
            else
            {
                sprintf(sz_TempValue, "%s", "");
            }
            jo_ProductDatum[arr_str_DcdcSettings[i].sz_key] = sz_TempValue;
            sprintf(arr_sz_SyncBuffer[i], "%s", sz_TempValue);
            ptr_sz_SyncBuffer[i] = arr_sz_SyncBuffer[i];
        }

        //Synchronize all other DCDCs to DCDC 0
        if (!bol_CommFailure)
        {
            int int_RequestSatuses[siz_NUM_DCDC_SETTING_KEYS];
            uint8_t u8_dcdcQty = CCA_Int_GetUnitQuantity(DCDC);
            for (uint8_t i = 1; i < u8_dcdcQty; i++)
            {
                int_rc = CCA_Icc_SetUnitConfiguration(CCA_sz_UNIT_TYPE_DCDC, i, siz_NUM_DCDC_SETTING_KEYS, 
                            arr_DcdcSettingKeys, (const char**)ptr_sz_SyncBuffer, int_RequestSatuses);
                if (int_rc != 0)
                {
                    LOG_INF("Failed to sync EMS Data for %s, index: %u, with Error: %i", CCA_sz_UNIT_TYPE_DCDC, i, int_rc);
                }
            }
        }
        else
        {
            LOG_INF("Failed to getEMSData for %s, index: %u, with Error: %i", CCA_sz_UNIT_TYPE_DCDC, 0, int_rc);
        }
        
        char sz_DcdcSettingsReport[CCA_u16_SETTINGS_JSON_DOC_SIZE];
        ArduinoJson::serializeJson(jd_DcdcSettingsReported, sz_DcdcSettingsReport);

        int_rc = CCA_AWS_PublishToTopic(&str_MqttContext, cca_sz_dcdc_update_topic, strlen(cca_sz_dcdc_update_topic),
                                        sz_DcdcSettingsReport, strlen(sz_DcdcSettingsReport));
    }
    else
    {
        LOG_ERR("Failed to initialize dcdc_settings shdadow.\n");
    }
    return (int_rc != EXIT_SUCCESS) ? int_rc : (bol_CommFailure) ? -EIO : 0;
}

ModScop int CCA_Dcdc_UpdateComponentData(uint8_t u8_UnitIndex)
{
    if (u8_UnitIndex != COD_u8_NULL_UNIT_ID && CCA_Int_ValidateUnitIndex(DCDC, u8_UnitIndex) != EXIT_SUCCESS)
    {
        return -EINVAL;
    }

    const char *sz_ProductUUID = cca_sz_thing_name;
    char sz_MqttPublishTopic[60];
    const uint8_t u8_DcdcQty = CCA_Int_GetUnitQuantity(DCDC);
    int int_rc[u8_DcdcQty] = {0};
    bool bol_CommFailure[u8_DcdcQty] = {false};

    uint8_t u8_EndIndex = (u8_UnitIndex == COD_u8_NULL_UNIT_ID) ? u8_DcdcQty : u8_UnitIndex + 1;
    uint8_t i = (u8_UnitIndex == COD_u8_NULL_UNIT_ID) ? 0 : u8_UnitIndex;
    for (i; i < u8_EndIndex; i++)
    {
        ArduinoJson::StaticJsonDocument<CCA_u16_COMPONENT_JSON_DOC_SIZE> jd_DcdcComponentData;
        sprintf(sz_MqttPublishTopic, "powersave/component/%s", cca_arr_dcdc_component_uuids[i]);
        // Build DCDC Component Data document
        jd_DcdcComponentData["id"] = cca_arr_dcdc_component_uuids[i];
        jd_DcdcComponentData["product_id"] = sz_ProductUUID;
        jd_DcdcComponentData["type"] = CCA_sz_UNIT_TYPE_DCDC;
        jd_DcdcComponentData["mfr"] = cca_sz_DCDC_MFR;
        bol_CommFailure[i] = false;
        uint32_t u32_DcdcParametersSize = CCA_DCDC_PARAMS_SIZE;
        int_rc[i] = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_DCDC, i, &str_DcdcParameters, &u32_DcdcParametersSize);
        if (int_rc[i] != 0 || u32_DcdcParametersSize != CCA_DCDC_PARAMS_SIZE)
        {
            LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i", 
                        CCA_sz_UNIT_TYPE_DCDC, i, u32_DcdcParametersSize, int_rc[i]);
            bol_CommFailure[i] = true;
        }
        //Populate the JSON Document with the DCDC Data
        CCA_Int_AppendToJsonDoc(jd_DcdcComponentData, arr_str_DcdcCompData, siz_NUM_COMP_KEYS, bol_CommFailure[i]);
        // Check for alarms, faults, and statuses
        ArduinoJson::JsonArray ja_Alarms = jd_DcdcComponentData.createNestedArray("alarms");
        ArduinoJson::JsonArray ja_Faults = jd_DcdcComponentData.createNestedArray("faults");
        ArduinoJson::JsonArray ja_Status = jd_DcdcComponentData.createNestedArray("status");
        CCA_Int_AppendCodesToJsonArray(ja_Alarms, arr_str_DcdcAlarmCodes, 
            siz_NUM_ALARM_KEYS, &str_DcdcParameters.codes);
        CCA_Int_AppendCodesToJsonArray(ja_Faults, arr_str_DcdcFaultCodes,
            siz_NUM_FAULT_KEYS, &str_DcdcParameters.codes);
        CCA_Int_AppendCodesToJsonArray(ja_Status, arr_str_DcdcStatusCodes,
            siz_NUM_STATUS_KEYS, &str_DcdcParameters.codes);

        char sz_DcdcComponentData[CCA_u16_COMPONENT_JSON_DOC_SIZE];
        ArduinoJson::serializeJson(jd_DcdcComponentData, sz_DcdcComponentData);
        // Validate the JSON prior to publish
        if (!jd_DcdcComponentData.isNull())
        {
            CCA_AWS_PublishToTopic(&str_MqttContext, sz_MqttPublishTopic, strlen(sz_MqttPublishTopic),
                sz_DcdcComponentData, strlen(sz_DcdcComponentData));
        }
        else
        {
            // Print the error messages
            LOG_ERR("ERROR: INVALID JSON during AWS_PublishComponentData()");
            LOG_ERR("%s\n", sz_DcdcComponentData);
        }
    }

    int update_rc = 0;
    for (uint8_t i = 0; i < u8_DcdcQty; i++)
    {
        if (int_rc[i] != 0)
        {
            update_rc = int_rc[i];
            break;
        }
        else if (bol_CommFailure[i])
        {
            update_rc = -EIO;
            break;
        }
    }
    return update_rc;
}

ModScop int CCA_Dcdc_GetProductData(ArduinoJson::JsonDocument& jd_ProductData, ArduinoJson::JsonArray& ja_Alarms,
                                    ArduinoJson::JsonArray& ja_Faults)
{
    uint32_t u32_DcdcParametersSize = CCA_DCDC_PARAMS_SIZE;
    int64_t arr_s64_ResponseSums[siz_NUM_PROD_KEYS] = {0};
    const uint8_t u8_DcdcQty = CCA_Int_GetUnitQuantity(DCDC);
    bool bol_CommFailure = false;
    int int_rc = -ENXIO;
    
    for (uint8_t i = 0; i < u8_DcdcQty; i++)
    {
        u32_DcdcParametersSize = CCA_DCDC_PARAMS_SIZE;
        memset(&str_DcdcParameters, 0, u32_DcdcParametersSize); //Clear the buffer before each call
        int_rc = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_DCDC, i, &str_DcdcParameters, &u32_DcdcParametersSize);
        if (int_rc != 0 || u32_DcdcParametersSize != CCA_DCDC_PARAMS_SIZE)
        {
            LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i", 
                        CCA_sz_UNIT_TYPE_DCDC, i, u32_DcdcParametersSize, int_rc);
            bol_CommFailure = true;
            break;
        }
        for (uint8_t j = 0; j < siz_NUM_PROD_KEYS; j++)
        {
            if (arr_str_DcdcProdData[j].ptr_s32_Val != NULL)
            {
                if (*arr_str_DcdcProdData[j].ptr_s32_Val != COD_s32_INVALID_DATA)
                {
                    arr_s64_ResponseSums[j] += *arr_str_DcdcProdData[j].ptr_s32_Val;
                }
                else
                {
                    arr_s64_ResponseSums[j] = COD_s32_INVALID_DATA;
                }
            }
            else if (arr_str_DcdcProdData[j].ptr_u32_Val != NULL)
            {
                arr_s64_ResponseSums[j] += *arr_str_DcdcProdData[j].ptr_u32_Val;
            }
            else
            {
                arr_s64_ResponseSums[j] = COD_s32_INVALID_DATA;
            }
        }
        //Add the DC/DC Alarms and Faults to the temp arrays
        if (!bol_CommFailure)
        {
            CCA_Int_AppendCodesToJsonArray(ja_Alarms, arr_str_DcdcAlarmCodes, 
                siz_NUM_ALARM_KEYS, &str_DcdcParameters.codes);
            CCA_Int_AppendCodesToJsonArray(ja_Faults, arr_str_DcdcFaultCodes,
                siz_NUM_FAULT_KEYS, &str_DcdcParameters.codes);
        }
    }
    //Append the summed values to the JSON document
    for (uint8_t j = 0; j < siz_NUM_PROD_KEYS; j++)
    {
        if (arr_str_DcdcProdData[j].ptr_s32_Val != NULL)
        {
            *arr_str_DcdcProdData[j].ptr_s32_Val = arr_s64_ResponseSums[j];
        }
        else if (arr_str_DcdcProdData[j].ptr_u32_Val != NULL)
        {
            *arr_str_DcdcProdData[j].ptr_u32_Val = arr_s64_ResponseSums[j];
        }
        else
        {
            LOG_ERR("ERROR: Invalid pointer in CCA_Dcdc_GetProductData()");
        }
    }
    CCA_Int_AppendToJsonDoc(jd_ProductData, arr_str_DcdcProdData, siz_NUM_PROD_KEYS, bol_CommFailure);
    return (int_rc < 0) ? int_rc : (bol_CommFailure) ? -EIO : 0;
}

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

//EOF
