#include "include/CCA_Gw.h"
#include "include/CCA_Internal.h"
#include "include/CCA_IccCalls.h"
#include "lion/log/log.h"

#define CCA_GW_PARAMS_SIZE sizeof(SCH_GwParameters_t)

LOG_MODULE_REGISTER(CCA_gw, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

FileScop SCH_GwParameters_t str_GwParameters;

FileScop const CCA_KeyToSchemaParamMap_t arr_str_GwProdData[] = {
    {"cab_temp", (int32_t *)&str_GwParameters.componentData.cab_temp, NULL},
    {"cab_hum", (int32_t *)&str_GwParameters.componentData.cab_hum, NULL}
};
FileScop const size_t siz_NUM_PROD_KEYS = sizeof(arr_str_GwProdData) / sizeof(arr_str_GwProdData[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_GwAlarmCodes[] = {
    #define X_GW(code) {#code, offsetof(SCH_GwCodes_t, alarm.code)},
    GW_CODES_ALARM_XMACRO
    #undef X_GW
};
FileScop const size_t siz_NUM_ALARM_KEYS = sizeof(arr_str_GwAlarmCodes) / sizeof(arr_str_GwAlarmCodes[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_GwFaultCodes[] = {
    #define X_GW(code) {#code, offsetof(SCH_GwCodes_t, fault.code)},
    GW_CODES_FAULT_XMACRO
    #undef X_GW
};
FileScop const size_t siz_NUM_FAULT_KEYS = sizeof(arr_str_GwFaultCodes) / sizeof(arr_str_GwFaultCodes[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_GwStatusCodes[] = {
    #define X_GW(code) {#code, offsetof(SCH_GwCodes_t, status.code)},
    GW_CODES_STATUS_XMACRO
    #undef X_GW
};
FileScop const size_t siz_NUM_STATUS_KEYS = sizeof(arr_str_GwStatusCodes) / sizeof(arr_str_GwStatusCodes[0]);

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

ModScop void CCA_Gw_Init(void)
{
    memset(&str_GwParameters, 0, CCA_GW_PARAMS_SIZE);
}

ModScop int CCA_Gw_UpdateComponentData(uint8_t u8_UnitIndex)
{
    (void)(u8_UnitIndex); //Suppress unused parameter warning
    return 0;
}

ModScop int CCA_Gw_GetProductData(ArduinoJson::JsonDocument& jd_ProductData, ArduinoJson::JsonArray& ja_Alarms,
                                    ArduinoJson::JsonArray& ja_Faults)
{
    (void)(ja_Alarms); //Suppress unused parameter warning

    uint32_t u32_GwParametersSize = CCA_GW_PARAMS_SIZE;
    const uint8_t u8_HvacQty = CCA_Int_GetUnitQuantity(HVAC);
    bool bol_CommFailure = false;
    int int_rc = -ENXIO;
    if (u8_HvacQty > 0)
    {
        // *Note: Using unit ID: 1 for these data calls due to gateway treating temp sensor as HVAC#2
        int_rc = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_GW, 1, &str_GwParameters, &u32_GwParametersSize);
    }
    if (int_rc != 0 || u32_GwParametersSize != CCA_GW_PARAMS_SIZE)
    {
        LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i", 
                    CCA_sz_UNIT_TYPE_GW, 1, u32_GwParametersSize, int_rc);
        bol_CommFailure = true;
    }
    //Populate the JSON Document with the GW Data, but only for the following keys:
    CCA_Int_AppendToJsonDoc(jd_ProductData, arr_str_GwProdData, siz_NUM_PROD_KEYS, bol_CommFailure);
    //Add the GW Alarms and Faults to the temp arrays
    u32_GwParametersSize = CCA_GW_PARAMS_SIZE;
    memset(&str_GwParameters, 0, u32_GwParametersSize); //Clear the buffer before using it again
    // *Note: We need to do a seperate call for the codes because those are only available at index 0.
    int_rc = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_GW, 0, &str_GwParameters, &u32_GwParametersSize);
    if (int_rc != 0 || u32_GwParametersSize != CCA_GW_PARAMS_SIZE)
    {
        LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i",
                    CCA_sz_UNIT_TYPE_GW, 0, u32_GwParametersSize, int_rc);
        bol_CommFailure = true;
    }
    else
    {
        CCA_Int_AppendCodesToJsonArray(ja_Alarms, arr_str_GwAlarmCodes, siz_NUM_ALARM_KEYS, &str_GwParameters.codes);
        CCA_Int_AppendCodesToJsonArray(ja_Faults, arr_str_GwFaultCodes, siz_NUM_FAULT_KEYS, &str_GwParameters.codes);
    }
    return (int_rc < 0) ? int_rc : (bol_CommFailure) ? -EIO : 0;
}

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

//EOF
