#include "include/CCA_Hvac.h"
#include "include/CCA_Internal.h"
#include "include/CCA_IccCalls.h"
#include "lion/log/log.h"

#define CCA_HVAC_PARAMS_SIZE sizeof(SCH_HvacParameters_t)

LOG_MODULE_REGISTER(CCA_hvac, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

FileScop SCH_HvacParameters_t str_HvacParameters;

FileScop const CCA_KeyToSchemaParamMap_t arr_str_HvacSettings[] = {
    #define X_HVAC(key) {#key, (int32_t*)&str_HvacParameters.settings.key, NULL},
    HVAC_SETTINGS_XMACRO
    #undef X_HVAC
};
FileScop const size_t siz_NUM_SETTINGS_KEYS = sizeof(arr_str_HvacSettings) / sizeof(arr_str_HvacSettings[0]);

FileScop const CCA_KeyToSchemaParamMap_t arr_str_HvacCompData[] = {
    #define X_HVAC(key) {#key, (int32_t*)&str_HvacParameters.componentData.key, NULL},
    HVAC_COMPONENT_DATA_XMACRO
    #undef X_HVAC
};
FileScop const size_t siz_NUM_COMP_KEYS = sizeof(arr_str_HvacCompData) / sizeof(arr_str_HvacCompData[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_HvacAlarmCodes[] = {
    #define X_HVAC(code) {#code, offsetof(SCH_HvacCodes_t, alarm.code)},
    HVAC_CODES_ALARM_XMACRO
    #undef X_HVAC
};
FileScop const size_t siz_NUM_ALARM_KEYS = sizeof(arr_str_HvacAlarmCodes) / sizeof(arr_str_HvacAlarmCodes[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_HvacFaultCodes[] = {
    #define X_HVAC(code) {#code, offsetof(SCH_HvacCodes_t, fault.code)},
    HVAC_CODES_FAULT_XMACRO
    #undef X_HVAC
};
FileScop const size_t siz_NUM_FAULT_KEYS = sizeof(arr_str_HvacFaultCodes) / sizeof(arr_str_HvacFaultCodes[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_HvacStatusCodes[] = {
    #define X_HVAC(code) {#code, offsetof(SCH_HvacCodes_t, status.code)},
    HVAC_CODES_STATUS_XMACRO
    #undef X_HVAC
};
FileScop const size_t siz_NUM_STATUS_KEYS = sizeof(arr_str_HvacStatusCodes) / sizeof(arr_str_HvacStatusCodes[0]);

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

ModScop void CCA_Hvac_Init(void)
{
    memset(&str_HvacParameters, 0, CCA_HVAC_PARAMS_SIZE);
}

ModScop int CCA_Hvac_InitShadows(void)
{
    bool bol_CommFailure = false;
    int int_rc = EXIT_SUCCESS;

    // Subscribe to HVAC_Settings Shadow sub-topics
    int_rc = CCA_AWS_SubscribeToShadowSubtopics(CCA_sz_HVAC_SHADOW_NAME, cca_sz_hvac_update_delta_topic, 
                                                cca_sz_hvac_update_accepted_topic, cca_sz_hvac_update_rejected_topic);
    // Publish HVAC Settings
    if (int_rc == EXIT_SUCCESS)
    {
        ArduinoJson::StaticJsonDocument<CCA_u16_SETTINGS_JSON_DOC_SIZE> jd_HvacSettingsReported;
        ArduinoJson::JsonObject jo_Product = jd_HvacSettingsReported["state"]["reported"].createNestedObject("product");
        jo_Product["id"] = cca_sz_thing_name;
        jo_Product["name"] = cca_sz_product_name;
        ArduinoJson::JsonObject jo_ProductDatum = jo_Product.createNestedObject("datum");
        
        uint32_t u32_HvacSettingsSize = CCA_HVAC_PARAMS_SIZE;
        int_rc = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_HVAC, 0, &str_HvacParameters, &u32_HvacSettingsSize);
        if (int_rc != EXIT_SUCCESS || u32_HvacSettingsSize != CCA_HVAC_PARAMS_SIZE)
        {
            LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i",
                        CCA_sz_UNIT_TYPE_HVAC, 0, u32_HvacSettingsSize, int_rc);
            bol_CommFailure = true;
        }
        //Populate the JSON Document with the HVAC settings
        CCA_Int_AppendToJsonObject(jo_ProductDatum, arr_str_HvacSettings, siz_NUM_SETTINGS_KEYS, bol_CommFailure);

        char sz_HvacSettingsReport[CCA_u16_SETTINGS_JSON_DOC_SIZE];
        ArduinoJson::serializeJson(jd_HvacSettingsReported, sz_HvacSettingsReport);
        int_rc = CCA_AWS_PublishToTopic(&str_MqttContext, cca_sz_hvac_update_topic, strlen(cca_sz_hvac_update_topic),
                                        sz_HvacSettingsReport, strlen(sz_HvacSettingsReport));
    }
    else
    {
        LOG_ERR("Failed to initialize hvac_settings shdadow.\n");
    }


    return (int_rc != EXIT_SUCCESS) ? int_rc : (bol_CommFailure) ? -EIO : 0;
}

ModScop int CCA_Hvac_UpdateComponentData(uint8_t u8_UnitIndex)
{
    if (u8_UnitIndex != COD_u8_NULL_UNIT_ID && CCA_Int_ValidateUnitIndex(HVAC, u8_UnitIndex) != EXIT_SUCCESS)
    {
        return -EINVAL;
    }

    const char *sz_ProductUUID = cca_sz_thing_name;
    char sz_MqttPublishTopic[60];
    const uint8_t u8_HvacQty = CCA_Int_GetUnitQuantity(HVAC);
    int int_rc[u8_HvacQty] = {0};
    bool bol_CommFailure[u8_HvacQty] = {false};

    uint8_t u8_EndIndex = (u8_UnitIndex == COD_u8_NULL_UNIT_ID) ? u8_HvacQty : u8_UnitIndex + 1;
    uint8_t i = (u8_UnitIndex == COD_u8_NULL_UNIT_ID) ? 0 : u8_UnitIndex;
    for (i; i < u8_EndIndex; i++)
    {
        ArduinoJson::StaticJsonDocument<CCA_u16_COMPONENT_JSON_DOC_SIZE> jd_HvacComponentData;
        sprintf(sz_MqttPublishTopic, "powersave/component/%s", cca_arr_hvac_component_uuids[i]);
        // Build HVAC Component Data document
        jd_HvacComponentData["id"] = cca_arr_hvac_component_uuids[i];
        jd_HvacComponentData["product_id"] = sz_ProductUUID;
        jd_HvacComponentData["type"] = CCA_sz_UNIT_TYPE_HVAC;
        bol_CommFailure[i] = false;
        uint32_t u32_HvacParametersSize = CCA_HVAC_PARAMS_SIZE;
        int_rc[i] = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_HVAC, i, &str_HvacParameters, &u32_HvacParametersSize);
        if (int_rc[i] != 0 || u32_HvacParametersSize != CCA_HVAC_PARAMS_SIZE)
        {
            LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i",
                        CCA_sz_UNIT_TYPE_HVAC, i, u32_HvacParametersSize, int_rc[i]);
            bol_CommFailure[i] = true;
        }
        //Populate the JSON Document with the HVAC Data
        CCA_Int_AppendToJsonDoc(jd_HvacComponentData, arr_str_HvacCompData, siz_NUM_COMP_KEYS, bol_CommFailure[i]);
        // Check for alarms, faults, and statuses
        ArduinoJson::JsonArray ja_Alarms = jd_HvacComponentData.createNestedArray("alarms");
        ArduinoJson::JsonArray ja_Faults = jd_HvacComponentData.createNestedArray("faults");
        ArduinoJson::JsonArray ja_Status = jd_HvacComponentData.createNestedArray("status");
        CCA_Int_AppendCodesToJsonArray(ja_Alarms, arr_str_HvacAlarmCodes, siz_NUM_ALARM_KEYS, &str_HvacParameters.codes);
        CCA_Int_AppendCodesToJsonArray(ja_Faults, arr_str_HvacFaultCodes, siz_NUM_FAULT_KEYS, &str_HvacParameters.codes);
        CCA_Int_AppendCodesToJsonArray(ja_Status, arr_str_HvacStatusCodes, siz_NUM_STATUS_KEYS, &str_HvacParameters.codes);

        char sz_HvacComponentData[CCA_u16_COMPONENT_JSON_DOC_SIZE];
        ArduinoJson::serializeJson(jd_HvacComponentData, sz_HvacComponentData);
        // Validate the JSON prior to publish
        if (!jd_HvacComponentData.isNull())
        {
            CCA_AWS_PublishToTopic(&str_MqttContext, sz_MqttPublishTopic, strlen(sz_MqttPublishTopic),
                sz_HvacComponentData, strlen(sz_HvacComponentData));
        }
        else
        {
            // Print the error messages
            LOG_ERR("ERROR: INVALID JSON during AWS_PublishComponentData()");
            LOG_ERR("%s\n", sz_HvacComponentData);
        }
    }
    
    int update_rc = 0;
    for (uint8_t i = 0; i < u8_HvacQty; i++)
    {
        if (int_rc[i] != 0)
        {
            update_rc = int_rc[i];
            break;
        }
        else if (bol_CommFailure[i])
        {
            update_rc = -EIO;
            break;
        }
    }
    return update_rc;
}

ModScop int CCA_Hvac_GetProductData(ArduinoJson::JsonDocument& jd_ProductData, ArduinoJson::JsonArray& ja_Alarms,
                                    ArduinoJson::JsonArray& ja_Faults)
{
    return 0; //We do not have any product data from the HVAC, this is gathered from the GW.
}

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

//EOF
