#include "include/CCA_Internal.h"
#include "include/CCA_IccCalls.h"
#include <lion/log/log.h>
#include <unistd.h>

LOG_MODULE_REGISTER(CCA_Internal, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

FileScop uint8_t cca_u8_DcdcQty = 0;
FileScop uint8_t cca_u8_GwQty   = 1;
FileScop uint8_t cca_u8_HvacQty = 1;
FileScop uint8_t cca_u8_PcsQty  = 1;
FileScop uint8_t cca_u8_PduQty  = 1;
FileScop uint8_t cca_u8_PmuQty  = 1;

/**
 * @brief Sets the system's timezone and localtime.
 * @param sz_TzValue Timezone to set to, e.g. "America/Denver"
 * 
 * @returns 0 on success,
 * @returns Error code on failure.
*/
FileScop int cca_int_SetTimezone(const char *sz_TzValue);

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

char cca_sz_mqtt_endpoint[CCA_u8_ENDPOINT_LENGTH];
char cca_sz_thing_name[CCA_u8_UUID_LENGTH];
char cca_sz_product_name[CCA_u8_PRODUCT_NAME_LENGTH];
char cca_arr_pcs_component_uuids[CCA_u8_MAX_COMPONENT_QTY][CCA_u8_UUID_LENGTH]; 
char cca_arr_pdu_component_uuids[CCA_u8_MAX_COMPONENT_QTY][CCA_u8_UUID_LENGTH];
char cca_arr_dcdc_component_uuids[CCA_u8_MAX_COMPONENT_QTY][CCA_u8_UUID_LENGTH];
char cca_arr_pmu_component_uuids[CCA_u8_MAX_COMPONENT_QTY][CCA_u8_UUID_LENGTH];
char cca_arr_hvac_component_uuids[CCA_u8_MAX_COMPONENT_QTY][CCA_u8_UUID_LENGTH];
char cca_sz_pcs_update_topic[CCA_u8_TOPIC_LENGTH]            = CCA_sz_SHADOW_PREFIX;
char cca_sz_pcs_update_delta_topic[CCA_u8_TOPIC_LENGTH]      = CCA_sz_SHADOW_PREFIX;
char cca_sz_pcs_update_accepted_topic[CCA_u8_TOPIC_LENGTH]   = CCA_sz_SHADOW_PREFIX;
char cca_sz_pcs_update_rejected_topic[CCA_u8_TOPIC_LENGTH]   = CCA_sz_SHADOW_PREFIX;
char cca_sz_dcdc_update_topic[CCA_u8_TOPIC_LENGTH]           = CCA_sz_SHADOW_PREFIX;
char cca_sz_dcdc_update_delta_topic[CCA_u8_TOPIC_LENGTH]     = CCA_sz_SHADOW_PREFIX;
char cca_sz_dcdc_update_accepted_topic[CCA_u8_TOPIC_LENGTH]  = CCA_sz_SHADOW_PREFIX;
char cca_sz_dcdc_update_rejected_topic[CCA_u8_TOPIC_LENGTH]  = CCA_sz_SHADOW_PREFIX;
char cca_sz_ems_update_topic[CCA_u8_TOPIC_LENGTH]            = CCA_sz_SHADOW_PREFIX;
char cca_sz_ems_update_delta_topic[CCA_u8_TOPIC_LENGTH]      = CCA_sz_SHADOW_PREFIX;
char cca_sz_ems_update_accepted_topic[CCA_u8_TOPIC_LENGTH]   = CCA_sz_SHADOW_PREFIX;
char cca_sz_ems_update_rejected_topic[CCA_u8_TOPIC_LENGTH]   = CCA_sz_SHADOW_PREFIX;
char cca_sz_pmu_update_topic[CCA_u8_TOPIC_LENGTH]            = CCA_sz_SHADOW_PREFIX;
char cca_sz_pmu_update_delta_topic[CCA_u8_TOPIC_LENGTH]      = CCA_sz_SHADOW_PREFIX;
char cca_sz_pmu_update_accepted_topic[CCA_u8_TOPIC_LENGTH]   = CCA_sz_SHADOW_PREFIX;
char cca_sz_pmu_update_rejected_topic[CCA_u8_TOPIC_LENGTH]   = CCA_sz_SHADOW_PREFIX;
char cca_sz_hvac_update_topic[CCA_u8_TOPIC_LENGTH]           = CCA_sz_SHADOW_PREFIX;
char cca_sz_hvac_update_delta_topic[CCA_u8_TOPIC_LENGTH]     = CCA_sz_SHADOW_PREFIX;
char cca_sz_hvac_update_accepted_topic[CCA_u8_TOPIC_LENGTH]  = CCA_sz_SHADOW_PREFIX;
char cca_sz_hvac_update_rejected_topic[CCA_u8_TOPIC_LENGTH]  = CCA_sz_SHADOW_PREFIX;
char cca_sz_version_update_topic[CCA_u8_TOPIC_LENGTH]        = CCA_sz_SHADOW_PREFIX;
char cca_sz_version_update_delta_topic[CCA_u8_TOPIC_LENGTH]  = CCA_sz_SHADOW_PREFIX;
char cca_sz_version_update_accepted_topic[CCA_u8_TOPIC_LENGTH] = CCA_sz_SHADOW_PREFIX;
char cca_sz_version_update_rejected_topic[CCA_u8_TOPIC_LENGTH] = CCA_sz_SHADOW_PREFIX;
char cca_sz_command_request_topic[CCA_u8_TOPIC_LENGTH]       = CCA_sz_COMMAND_REQUEST_PREFIX;
char cca_sz_command_response_topic[CCA_u8_TOPIC_LENGTH]      = CCA_sz_COMMAND_RESPONSE_PREFIX;
char cca_sz_alerts_topic[CCA_u8_TOPIC_LENGTH]                = CCA_sz_ALERTS_TOPIC_PREFIX;
char cca_sz_ems_parallel_cfg[CCA_u8_PARALLEL_CFG_LENGTH]     = CCA_sz_EMS_PARALLEL_DISABLED;

const char cca_arr_PCS_SERIALS[][16] = { "PWS003210922001", "PWS003210922002" };
const char *cca_sz_PCS_MFR = "Sinexcel Inc.";
const char cca_arr_PDU_SERIALS[][16] = {""};
const char *cca_sz_PDU_MFR = "nruit";
const char *cca_sz_PDU_MODEL = "91000012";
const char *cca_sz_DCDC_MFR = "Sinexcel Inc.";
const char *cca_sz_PMU_MFR = "lion";

ModScop int CCA_Int_UnitTypeStringToEnum(const char *sz_UnitType, unit_type_t *ptr_UnitType)
{
    int int_rc = EXIT_SUCCESS;
    if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_PCS) == 0)
    {
        *ptr_UnitType = PCS;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_PDU) == 0)
    {
        *ptr_UnitType = PDU;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_PMU) == 0)
    {
        *ptr_UnitType = PMU;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_DCDC) == 0)
    {
        *ptr_UnitType = DCDC;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_GW) == 0)
    {
        *ptr_UnitType = GW;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_HVAC) == 0)
    {
        *ptr_UnitType = HVAC;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_EMS) == 0)
    {
        *ptr_UnitType = EMS;
    }
    else
    {
        int_rc = EXIT_FAILURE;
    }
    return int_rc;
}

ModScop int CCA_Int_InitUnitQuantities(void)
{
    const char *sz_Keys[] = { "dcdc_qty", "hvac_qty", "pcs_qty", "pdu_qty", "pmu_qty" };
    const uint8_t NUM_KEYS = sizeof(sz_Keys) / sizeof(sz_Keys[0]);
    uint8_t *u8_Qtys[] = { &cca_u8_DcdcQty, &cca_u8_HvacQty, &cca_u8_PcsQty, &cca_u8_PduQty, &cca_u8_PmuQty };
    char* ResponseBuffer[NUM_KEYS] = {NULL};
    // Retrieve the device quantities from the EMS
    int int_rc = CCA_Icc_GetEMSConfiguration(CCA_sz_UNIT_TYPE_EMS, COD_u8_NULL_UNIT_ID, NUM_KEYS, sz_Keys, ResponseBuffer);
    if (int_rc != 0)
    {
        LOG_ERR("Failed to get device quantities from the EMS, with Error: %i", int_rc);
        return EXIT_FAILURE;
    }
    for (uint8_t i = 0; i < NUM_KEYS; i++)
    {
        //Set each device quantity, skip and leave default if we didnt get anything from the EMS
        if (ResponseBuffer[i] != NULL)
        {
            *u8_Qtys[i] = atoi(ResponseBuffer[i]);
        }
    }
    return EXIT_SUCCESS;
}

ModScop uint8_t CCA_Int_GetUnitQuantity(unit_type_t enm_Unit)
{
    switch (enm_Unit)
    {
        case DCDC:
            return cca_u8_DcdcQty;
        case GW:
            return cca_u8_GwQty;
        case HVAC:
            return cca_u8_HvacQty;
        case PCS:
            return cca_u8_PcsQty;
        case PDU:
            return cca_u8_PduQty;
        case PMU:
            return cca_u8_PmuQty;
        default:
            return 0;
    }
}

ModScop uint8_t CCA_Int_GetUnitQuantity_sz(const char *sz_UnitType)
{
    if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_DCDC) == EXIT_SUCCESS)
    {
        return cca_u8_DcdcQty;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_GW) == EXIT_SUCCESS)
    {
        return cca_u8_GwQty;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_HVAC) == EXIT_SUCCESS)
    {
        return cca_u8_HvacQty;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_PCS) == EXIT_SUCCESS)
    {
        return cca_u8_PcsQty;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_PDU) == EXIT_SUCCESS)
    {
        return cca_u8_PduQty;
    }
    else if (strcmp(sz_UnitType, CCA_sz_UNIT_TYPE_PMU) == EXIT_SUCCESS)
    {
        return cca_u8_PmuQty;
    }
    else
    {
        return 0;
    }
}

ModScop void CCA_Int_AppendToJsonDoc(ArduinoJson::JsonDocument& jd_Doc, const CCA_KeyToSchemaParamMap_t *arr_KeysToParams, 
                                        size_t siz_Array, bool bol_CommFailure)
{
    ArduinoJson::JsonObject jo_DocAsObj = jd_Doc.as<ArduinoJson::JsonObject>();
    CCA_Int_AppendToJsonObject(jo_DocAsObj, arr_KeysToParams, siz_Array, bol_CommFailure);
}

ModScop void CCA_Int_AppendToJsonObject(ArduinoJson::JsonObject& jo_Obj, const CCA_KeyToSchemaParamMap_t *arr_KeysToParams, 
                                        size_t siz_Array, bool bol_CommFailure)
{
    for (size_t i = 0; i < siz_Array; i++)
    {
        char tempValue[MAX_VALUE_SIZE];
        if (arr_KeysToParams[i].ptr_s32_Val != NULL && !bol_CommFailure)
        {
            if (*(arr_KeysToParams[i].ptr_s32_Val) != COD_s32_INVALID_DATA)
            {
                sprintf(tempValue, "%i", *(arr_KeysToParams[i].ptr_s32_Val));
            }
            else
            {
                sprintf(tempValue, "%s", "");
            }
        }
        else if (arr_KeysToParams[i].ptr_u32_Val != NULL && !bol_CommFailure)
        {
            sprintf(tempValue, "%u", *(arr_KeysToParams[i].ptr_u32_Val));
        }
        else
        {
            sprintf(tempValue, "%s", "");
        }
        jo_Obj[arr_KeysToParams[i].sz_key] = tempValue;
    }
}

ModScop void CCA_Int_AppendCodesToJsonArray(ArduinoJson::JsonArray& ja_Arr, const CCA_KeyToSchemaCodeMap_t *arr_KeysToCodes, 
                                            size_t siz_Array, void *vptr_CodesStruct)
{
    for (size_t i = 0; i < siz_Array; i++)
    {
        if (*(bool*)((char*)vptr_CodesStruct + arr_KeysToCodes[i].siz_StrOffset))
        {
            ja_Arr.add(arr_KeysToCodes[i].sz_code);
        }
    }
}

ModScop int CCA_Int_AppendTouSettingsToJsonObject(ArduinoJson::JsonObject& jo_Obj)
{
    StackScop ArduinoJson::JsonObject jo_TouProf = jo_Obj.createNestedObject(CCA_sz_TOU_KEY);
    
    //Retrieve rethe entire Tou Settings struct buffer
    StackScop SCH_EmsTouSettings_t str_TouSettings = {0};
    StackScop uint32_t u32_TouDataSize = sizeof(SCH_EmsTouSettings_t);
    StackScop int int_rc = CCA_Icc_GetBufferedData(IAL_sz_TYPE_TOU, 0, &str_TouSettings, &u32_TouDataSize);
    if (int_rc != EXIT_SUCCESS || u32_TouDataSize != sizeof(SCH_EmsTouSettings_t))
    {
        LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i",
                    IAL_sz_TYPE_TOU, 0, u32_TouDataSize, int_rc);
    }
    else
    {
        //Add the Time of Use Profile Settings
        jo_TouProf["qty"] = str_TouSettings.u8_ProfileQty;
        //Add a nested profile object for each profile
        for (uint8_t i = 0; i < str_TouSettings.u8_ProfileQty; i++)
        {
            char sz_temp[5];
            sprintf(sz_temp, "%u", i);
            ArduinoJson::JsonObject jo_Profile = jo_TouProf.createNestedObject(sz_temp);

            //Add the effective dates for the profile
            ArduinoJson::JsonArray ja_EffDates = jo_Profile.createNestedArray("eff_dates");
            for (uint8_t k = 0; k < str_TouSettings.arr_str_Profiles[i].u8_EffDatesQty; k++)
            {
                ArduinoJson::JsonObject jo_DateRange = ja_EffDates.createNestedObject();
                const char *sz_Months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", 
                                            "Nov", "Dec"};
                SCH_EmsTouEffDates_t *ptr_str_DateRange = &str_TouSettings.arr_str_Profiles[i].arr_str_EffDates[k];
                char sz_Temp[CCA_u8_STRING_BUFFER_SIZE];
                jo_DateRange["start_month"] = sz_Months[ptr_str_DateRange->u8_StartMonth];
                jo_DateRange["start_day"] = ptr_str_DateRange->u8_StartDay;
                jo_DateRange["stop_month"] = sz_Months[ptr_str_DateRange->u8_StopMonth];
                jo_DateRange["stop_day"] = ptr_str_DateRange->u8_StopDay;
            }

            //Add all the days objects and their array of timeslots
            const char *sz_DaysOfTheWeek[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
            for (uint8_t j = 0; j < sizeof(sz_DaysOfTheWeek)/sizeof(sz_DaysOfTheWeek[0]); j++)
            {
                if (str_TouSettings.arr_str_Profiles[i].arr_str_Days[j].u8_TimeSlotQty == 0)
                {
                    continue; //Skip days with no timeslots
                }

                ArduinoJson::JsonArray ja_DayOfTheWeek = jo_Profile.createNestedArray(sz_DaysOfTheWeek[j]);

                for (uint8_t k = 0; k < str_TouSettings.arr_str_Profiles[i].arr_str_Days[j].u8_TimeSlotQty; k++)
                {
                    ArduinoJson::JsonObject jo_TimeSlot = ja_DayOfTheWeek.createNestedObject();
                    SCH_EmsTouTimeslot_t *str_TimeSlot = &str_TouSettings.arr_str_Profiles[i].arr_str_Days[j].arr_str_TimeSlots[k];
                    char sz_Temp[CCA_u8_STRING_BUFFER_SIZE];
                    sprintf(sz_Temp, "%02hu:%02hu:%02hu", str_TimeSlot->u8_Hours, str_TimeSlot->u8_Minutes, str_TimeSlot->u8_Seconds);
                    jo_TimeSlot["start_time"] = sz_Temp;
                    sprintf(sz_Temp, "%u", str_TimeSlot->u8_MinSOC);
                    jo_TimeSlot["min_soc"] = sz_Temp;
                    sprintf(sz_Temp, "%u", str_TimeSlot->u8_GridChg);
                    jo_TimeSlot["grid_ch"] = sz_Temp;
                }
            }
        }
    }
    return int_rc;
}

ModScop int CCA_Int_SetCCUSettings(uint8_t u8_numberOfKeys, const char **sz_arr_keys, const char **sz_arr_values)
{
    int int_RC;
    for (uint8_t i = 0; i < u8_numberOfKeys; i++)
    {   
        if (strcmp(sz_arr_keys[i], "timezone") == 0)
        {
            int_RC = cca_int_SetTimezone(sz_arr_values[i]);
            if (int_RC < 0)
            {
                return int_RC;
            }
        }
    }
    return 0;
}

int CCA_Int_ValidateUnitType(const char *sz_UnitType, unit_type_t *ptr_UnitType)
{
    if (strncmp(sz_UnitType, CCA_sz_UNIT_TYPE_PCS, strlen(CCA_sz_UNIT_TYPE_PCS)) == EXIT_SUCCESS)
    {
        *ptr_UnitType = PCS;
    }
    else if (strncmp(sz_UnitType, CCA_sz_UNIT_TYPE_PDU, strlen(CCA_sz_UNIT_TYPE_PDU)) == EXIT_SUCCESS)
    {
        *ptr_UnitType = PDU;
    }
    else if (strncmp(sz_UnitType, CCA_sz_UNIT_TYPE_PMU, strlen(CCA_sz_UNIT_TYPE_PMU)) == EXIT_SUCCESS)
    {
        *ptr_UnitType = PMU;
    }
    else if (strncmp(sz_UnitType, CCA_sz_UNIT_TYPE_DCDC, strlen(CCA_sz_UNIT_TYPE_DCDC)) == EXIT_SUCCESS)
    {
        *ptr_UnitType = DCDC;
    }
    else if (strncmp(sz_UnitType, CCA_sz_UNIT_TYPE_GW, strlen(CCA_sz_UNIT_TYPE_GW)) == EXIT_SUCCESS)
    {
        *ptr_UnitType = GW;
    }
    else if (strncmp(sz_UnitType, CCA_sz_UNIT_TYPE_HVAC, strlen(CCA_sz_UNIT_TYPE_HVAC)) == EXIT_SUCCESS)
    {
        *ptr_UnitType = HVAC;
    }
    else if (strncmp(sz_UnitType, CCA_sz_UNIT_TYPE_EMS, strlen(CCA_sz_UNIT_TYPE_EMS)) == EXIT_SUCCESS)
    {
        *ptr_UnitType = EMS;
    }
    else
    {
        LOG_ERR("Error: Unknown device type: %s", sz_UnitType);
        return -ENODEV;
    }
    return EXIT_SUCCESS;
}

int CCA_Int_ValidateUnitIndex(unit_type_t enm_UnitType, uint32_t u32_UnitIndex)
{
    switch (enm_UnitType)
    {
        case PCS:
            if (u32_UnitIndex >= cca_u8_PcsQty)
            {
                LOG_ERR("Invalid PCS index: %u", u32_UnitIndex);
                return -ENXIO;
            }
            break;
        case PDU:
            if (u32_UnitIndex >= cca_u8_PduQty)
            {
                LOG_ERR("Invalid PDU index: %u", u32_UnitIndex);
                return -ENXIO;
            }
            break;
        case PMU:
            if (u32_UnitIndex >= cca_u8_PmuQty)
            {
                LOG_ERR("Invalid PMU index: %u", u32_UnitIndex);
                return -ENXIO;
            }
            break;
        case DCDC:
            if (u32_UnitIndex >= cca_u8_DcdcQty)
            {
                LOG_ERR("Invalid DCDC index: %u", u32_UnitIndex);
                return -ENXIO;
            }
            break;
        case GW:
            if (u32_UnitIndex >= CCA_u8_MAX_GW_QTY)
            {
                LOG_ERR("Invalid GW index: %u", u32_UnitIndex);
                return -ENXIO;
            }
            break;
        case HVAC:
            if (u32_UnitIndex >= cca_u8_HvacQty)
            {
                LOG_ERR("Invalid HVAC index: %u", u32_UnitIndex);
                return -ENXIO;
            }
            break;
        case EMS:
            if (u32_UnitIndex >= CCA_u8_MAX_EMS_QTY)
            {
                LOG_ERR("Invalid EMS index: %u", u32_UnitIndex);
                return -ENXIO;
            }
        default:
            break;
    }
    return EXIT_SUCCESS;
}

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

FileScop int cca_int_SetTimezone(const char *sz_TzValue)
{
    LOG_DBG("Setting timezone: %s", sz_TzValue);
    StackScop FILE *fp = fopen("/data/etc/timezone", "w");
    if (fp == NULL || fprintf(fp, "%s\n", sz_TzValue) < 0)
    {
        LOG_ERR("Unable to write \"%s\" to /data/etc/timezone", sz_TzValue);
        return -EROFS;
    }
    if(fclose(fp) != 0)
    {
        LOG_ERR("Unable to close file (/data/etc/timezone)");
        return -EIO;
    }

    StackScop const char* sz_SymlinkPath = "/data/etc/localtime";
    StackScop const char *sz_ZoneInfoPath = "/usr/share/zoneinfo/";
    StackScop size_t siz_TzFullPathLen = strlen(sz_ZoneInfoPath) + strlen(sz_TzValue) + 1;
    StackScop char sz_TzFullPath[siz_TzFullPathLen];
    sprintf(sz_TzFullPath, "%s%s", sz_ZoneInfoPath, sz_TzValue);

    LOG_DBG("Setting localtime: %s", sz_TzFullPath);
    if (unlink(sz_SymlinkPath) < 0)
    {
        LOG_ERR("Unable to remove symlink");
        return -errno;
    }
    if (symlink(sz_TzFullPath, sz_SymlinkPath) < 0) {
        LOG_ERR("Unable to set symlink");
        return -errno;
    }

    return 0;
}

//EOF
