#include "include/CCA_Pcs.h"
#include "include/CCA_Internal.h"
#include "include/CCA_IccCalls.h"
#include "lion/log/log.h"


#define CCA_PCS_PARAMS_SIZE sizeof(SCH_PcsParameters_t)


LOG_MODULE_REGISTER(CCA_pcs, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

FileScop SCH_PcsParameters_t str_PcsParameters;

FileScop const CCA_KeyToSchemaParamMap_t arr_str_PcsSettings[] = {
    #define X_PCS(param) {#param, (int32_t*)&str_PcsParameters.settings.param, NULL},
    PCS_SETTINGS_XMACRO
    #undef X_PCS
    #define X_PCS(param) {#param, (int32_t*)&str_PcsParameters.deviceInfo.param, NULL},
    PCS_DEVICE_INFO_XMACRO
    #undef X_PCS
};
FileScop const size_t siz_NUM_PCS_SETTING_KEYS = sizeof(arr_str_PcsSettings) / sizeof(arr_str_PcsSettings[0]);

FileScop const char *arr_sz_PcsUnitInfoKeys[] = { "ip_addr", "netmask", "gw_ip_addr", "mac_addr" };
FileScop const uint8_t u8_NUM_PCS_UNIT_INFO_KEYS = sizeof(arr_sz_PcsUnitInfoKeys) / sizeof(arr_sz_PcsUnitInfoKeys[0]);

FileScop const CCA_KeyToSchemaParamMap_t arr_str_PcsProdData[] = {
    {"inv_p", (int32_t *)&str_PcsParameters.componentData.inv_p, NULL},
    {"inv_q", (int32_t *)&str_PcsParameters.componentData.inv_q, NULL},
    {"inv_s", (int32_t *)&str_PcsParameters.componentData.inv_s, NULL},
    {"inv_pf", (int32_t *)&str_PcsParameters.componentData.inv_pf, NULL},
    {"dc_p", (int32_t *)&str_PcsParameters.componentData.dc_p, NULL},
    {"ac_dch_kwh_total", (int32_t *)&str_PcsParameters.componentData.ac_dch_kwh_total, NULL},
    {"ac_ch_kwh_total", (int32_t *)&str_PcsParameters.componentData.ac_ch_kwh_total, NULL},
    {"ac_dch_kwh_day", (int32_t *)&str_PcsParameters.componentData.ac_dch_kwh_day, NULL},
    {"ac_ch_kwh_day", (int32_t *)&str_PcsParameters.componentData.ac_ch_kwh_day, NULL}
};
FileScop const size_t siz_NUM_PROD_KEYS = sizeof(arr_str_PcsProdData) / sizeof(arr_str_PcsProdData[0]);

FileScop const CCA_KeyToSchemaParamMap_t arr_str_pcsCompData[] = {
    #define X_PCS(param) {#param, (int32_t*)&str_PcsParameters.componentData.param, NULL},
    PCS_COMPONENT_DATA_XMACRO
    #undef X_PCS
};
FileScop const size_t siz_NUM_COMP_KEYS = sizeof(arr_str_pcsCompData) / sizeof(arr_str_pcsCompData[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_PcsAlarmCodes[] = {
    #define X_PCS(code) {#code, offsetof(SCH_PcsCodes_t, alarm.code)},
    PCS_CODES_ALARM_XMACRO
    #undef X_PCS
};
FileScop const size_t siz_NUM_ALARM_KEYS = sizeof(arr_str_PcsAlarmCodes) / sizeof(arr_str_PcsAlarmCodes[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_PcsFaultCodes[] = {
    #define X_PCS(code) {#code, offsetof(SCH_PcsCodes_t, fault.code)},
    PCS_CODES_FAULT_XMACRO
    #undef X_PCS
};
FileScop const size_t siz_NUM_FAULT_KEYS = sizeof(arr_str_PcsFaultCodes) / sizeof(arr_str_PcsFaultCodes[0]);

FileScop const CCA_KeyToSchemaCodeMap_t arr_str_PcsStatusCodes[] = {
    #define X_PCS(code) {#code, offsetof(SCH_PcsCodes_t, status.code)},
    PCS_CODES_STATUS_XMACRO
    #undef X_PCS
};
FileScop const size_t siz_NUM_STATUS_KEYS = sizeof(arr_str_PcsStatusCodes) / sizeof(arr_str_PcsStatusCodes[0]);

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

ModScop void CCA_Pcs_Init(void)
{
    memset(&str_PcsParameters, 0, CCA_PCS_PARAMS_SIZE);
}

ModScop int CCA_Pcs_InitShadows(void)
{
    bool bol_CommFailure = false;
    int int_rc = EXIT_SUCCESS;

    // Subscribe to PCS_Settings Shadow sub-topics
    int_rc = CCA_AWS_SubscribeToShadowSubtopics(CCA_sz_PCS_SHADOW_NAME, cca_sz_pcs_update_delta_topic, 
                                                cca_sz_pcs_update_accepted_topic, cca_sz_pcs_update_rejected_topic);
    // Publish PCS Settings
    if (int_rc == EXIT_SUCCESS)
    {
        ArduinoJson::StaticJsonDocument<CCA_u16_SETTINGS_JSON_DOC_SIZE> jd_PcsSettingsReported;
        ArduinoJson::JsonObject jo_Product = jd_PcsSettingsReported["state"]["reported"].createNestedObject("product");
        jo_Product["id"] = cca_sz_thing_name;
        jo_Product["name"] = cca_sz_product_name;
        ArduinoJson::JsonObject jo_ProductDatum = jo_Product.createNestedObject("datum");
        
        uint32_t u32_PcsParametersSize = CCA_PCS_PARAMS_SIZE;
        int_rc = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_PCS, COD_u8_NULL_UNIT_ID, &str_PcsParameters, 
                                            &u32_PcsParametersSize);
        if (int_rc != EXIT_SUCCESS || u32_PcsParametersSize != CCA_PCS_PARAMS_SIZE)
        {
            LOG_INF("Failed to getEMSData for %s, index: 0, bytes: %d, with Error: %i", 
                    CCA_sz_UNIT_TYPE_PCS, u32_PcsParametersSize, int_rc);
            bol_CommFailure = true;
        }
        //Populate the JSON Document with the PCS Settings
        CCA_Int_AppendToJsonObject(jo_ProductDatum, arr_str_PcsSettings, siz_NUM_PCS_SETTING_KEYS, bol_CommFailure);

        // Get Unit Info Settings
        char *arr_sz_PcsUnitSettingsValues[u8_NUM_PCS_UNIT_INFO_KEYS] = { NULL };
        if (CCA_Icc_GetUnitInfo(CCA_sz_UNIT_TYPE_PCS, COD_u8_NULL_UNIT_ID, u8_NUM_PCS_UNIT_INFO_KEYS, 
                                arr_sz_PcsUnitInfoKeys, (char**)arr_sz_PcsUnitSettingsValues) != EXIT_SUCCESS) 
        {
            LOG_INF("ERROR: Failed to CCA_Icc_GetUnitInfo for %s, index: %u", CCA_sz_UNIT_TYPE_PCS, COD_u8_NULL_UNIT_ID);
            bol_CommFailure = true;
        }
        for (uint8_t i = 0; i < u8_NUM_PCS_UNIT_INFO_KEYS; i++)
        {
            if (arr_sz_PcsUnitSettingsValues[i] != NULL)
            {
                jo_ProductDatum[arr_sz_PcsUnitInfoKeys[i]] = arr_sz_PcsUnitSettingsValues[i];
            }
            else 
            {
                jo_ProductDatum[arr_sz_PcsUnitInfoKeys[i]] = "";
            }
        }

        char sz_PcsSettingsReport[CCA_u16_SETTINGS_JSON_DOC_SIZE];
        ArduinoJson::serializeJson(jd_PcsSettingsReported, sz_PcsSettingsReport);
        int_rc = CCA_AWS_PublishToTopic(&str_MqttContext, cca_sz_pcs_update_topic, strlen(cca_sz_pcs_update_topic),
                                            sz_PcsSettingsReport, strlen(sz_PcsSettingsReport));
    }
    else
    {
        LOG_ERR("ERROR: Failed to initialize %s shdadow.", CCA_sz_PCS_SHADOW_NAME);
    }
    return (int_rc != EXIT_SUCCESS) ? int_rc : (bol_CommFailure) ? -EIO : 0;
}

ModScop int CCA_Pcs_UpdateComponentData(uint8_t u8_UnitIndex)
{
    if (u8_UnitIndex != COD_u8_NULL_UNIT_ID && CCA_Int_ValidateUnitIndex(PCS, u8_UnitIndex) != EXIT_SUCCESS)
    {
        return -EINVAL;
    }

    const char *sz_PcsModel = "PWS2-30P-NA";
    const char *sz_ProductUUID = cca_sz_thing_name;
    char sz_MqttPublishTopic[60];
    const uint8_t u8_PcsQty = CCA_Int_GetUnitQuantity(PCS);
    int int_rc[u8_PcsQty] = {0};
    bool bol_CommFailure[u8_PcsQty] = {false};

    uint8_t u8_EndIndex = (u8_UnitIndex == COD_u8_NULL_UNIT_ID) ? u8_PcsQty : u8_UnitIndex + 1;
    uint8_t i = (u8_UnitIndex == COD_u8_NULL_UNIT_ID) ? 0 : u8_UnitIndex;
    for (i; i < u8_EndIndex; i++)
    {
        ArduinoJson::StaticJsonDocument<CCA_u16_COMPONENT_JSON_DOC_SIZE> jd_PcsComponentData;
        sprintf(sz_MqttPublishTopic, "powersave/component/%s", cca_arr_pcs_component_uuids[i]);
        // Build PCS Component Data document
        jd_PcsComponentData["id"] = cca_arr_pcs_component_uuids[i];
        jd_PcsComponentData["product_id"] = sz_ProductUUID;
        jd_PcsComponentData["type"] = CCA_sz_UNIT_TYPE_PCS;
        jd_PcsComponentData["mfr"] = cca_sz_PCS_MFR;
        bol_CommFailure[i] = false;
        uint32_t u32_PcsParametersSize = CCA_PCS_PARAMS_SIZE;
        int_rc[i] = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_PCS, i, &str_PcsParameters, &u32_PcsParametersSize);
        if (int_rc[i] != 0 || u32_PcsParametersSize != CCA_PCS_PARAMS_SIZE)
        {
            LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i", 
                        CCA_sz_UNIT_TYPE_PCS, i, u32_PcsParametersSize, int_rc[i]);
            bol_CommFailure[i] = true;
        }
        //Populate the JSON Document with the PCS Data  
        CCA_Int_AppendToJsonDoc(jd_PcsComponentData, arr_str_pcsCompData, siz_NUM_COMP_KEYS, bol_CommFailure[i]);
        // Check for alarms, faults, and statuses
        ArduinoJson::JsonArray ja_Alarms = jd_PcsComponentData.createNestedArray("alarms");
        ArduinoJson::JsonArray ja_Faults = jd_PcsComponentData.createNestedArray("faults");
        ArduinoJson::JsonArray ja_Status = jd_PcsComponentData.createNestedArray("status");
        CCA_Int_AppendCodesToJsonArray(ja_Alarms, arr_str_PcsAlarmCodes, siz_NUM_ALARM_KEYS, &str_PcsParameters.codes);
        CCA_Int_AppendCodesToJsonArray(ja_Faults, arr_str_PcsFaultCodes, siz_NUM_FAULT_KEYS, &str_PcsParameters.codes);
        CCA_Int_AppendCodesToJsonArray(ja_Status, arr_str_PcsStatusCodes, siz_NUM_STATUS_KEYS, &str_PcsParameters.codes);

        char sz_PcsComponentData[CCA_u16_COMPONENT_JSON_DOC_SIZE];
        ArduinoJson::serializeJson(jd_PcsComponentData, sz_PcsComponentData);
        // Validate the JSON prior to publish
        if (!jd_PcsComponentData.isNull())
        {
            CCA_AWS_PublishToTopic(&str_MqttContext, sz_MqttPublishTopic, strlen(sz_MqttPublishTopic),
                           sz_PcsComponentData, strlen(sz_PcsComponentData));
        }
        else
        {
            // Print the error messages
            LOG_ERR("ERROR: INVALID JSON during AWS_PublishComponentData()");
            LOG_ERR("%s\n", sz_PcsComponentData);
        }
    }
    
    int update_rc = 0;
    for (uint8_t i = 0; i < u8_PcsQty; i++)
    {
        if (int_rc[i] != 0)
        {
            update_rc = int_rc[i];
            break;
        }
        else if (bol_CommFailure[i])
        {
            update_rc = -EIO;
            break;
        }
    }
    return update_rc;
}

ModScop int CCA_Pcs_GetProductData(ArduinoJson::JsonDocument& jd_ProductData, ArduinoJson::JsonArray& ja_Alarms,
                                    ArduinoJson::JsonArray& ja_Faults)
{
    uint32_t u32_PcsParametersSize = CCA_PCS_PARAMS_SIZE;
    const uint8_t u8_PcsQty = CCA_Int_GetUnitQuantity(PCS);
    bool bol_CommFailure = false;
    int int_rc = -ENXIO;
    if (u8_PcsQty > 0)
    {
        int_rc = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_PCS, COD_u8_NULL_UNIT_ID, &str_PcsParameters, &u32_PcsParametersSize);
    }
    if (int_rc != 0 || u32_PcsParametersSize != CCA_PCS_PARAMS_SIZE)
    {
        LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i", 
                    CCA_sz_UNIT_TYPE_PCS, COD_u8_NULL_UNIT_ID, u32_PcsParametersSize, int_rc);
        bol_CommFailure = true;
    }

    //Populate the JSON Document with the PCS Data, but only for the following keys:
    CCA_Int_AppendToJsonDoc(jd_ProductData, arr_str_PcsProdData, siz_NUM_PROD_KEYS, bol_CommFailure);
    //Add the PCS Alarms and Faults to the temp arrays
    if (!bol_CommFailure)
    {
        CCA_Int_AppendCodesToJsonArray(ja_Alarms, arr_str_PcsAlarmCodes, siz_NUM_ALARM_KEYS, &str_PcsParameters.codes);
        CCA_Int_AppendCodesToJsonArray(ja_Faults, arr_str_PcsFaultCodes, siz_NUM_FAULT_KEYS, &str_PcsParameters.codes);
    }
    return (int_rc < 0) ? int_rc : (bol_CommFailure) ? -EIO : 0;
}

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

//EOF
