#include "include/CCA_Pmu.h"
#include "include/CCA_Internal.h"
#include "include/CCA_IccCalls.h"
#include "lion/log/log.h"

#define CCA_PMU_PARAMS_SIZE sizeof(SCH_PmuParameters_t)

LOG_MODULE_REGISTER(CCA_pmu, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

#define CCA_PMU_u16_SETTINGS_JSON_DOC_SIZE 2048

FileScop SCH_PmuParameters_t str_PmuParameters;

FileScop const char *arr_sz_PmuSettings[] = {   
    "grid_ct_a_rat","grid_ct_a_sf","grid_ct_a_ofs","grid_ct_b_rat","grid_ct_b_sf",
    "grid_ct_b_ofs","grid_ct_c_rat","grid_ct_c_sf","grid_ct_c_ofs","grid_vt_a_sf",
    "grid_vt_a_ofs","load_ct_a_rat","load_ct_a_sf","load_ct_a_ofs","load_ct_b_rat",
    "load_ct_b_sf","load_ct_b_ofs","load_ct_c_rat","load_ct_c_sf","load_ct_c_ofs",
    "load_vt_a_sf","load_vt_a_ofs","load_vt_b_sf","load_vt_b_ofs","load_vt_c_sf",
    "load_vt_c_ofs"
};
FileScop size_t siz_NUM_SETTINGS_KEYS = sizeof(arr_sz_PmuSettings) / sizeof(arr_sz_PmuSettings[0]);

FileScop const CCA_KeyToSchemaParamMap_t arr_str_PmuProdData[] = {
    {"grid_p", (int32_t *)&str_PmuParameters.componentData.grid_p, NULL},
    {"grid_v", (int32_t *)&str_PmuParameters.componentData.grid_v, NULL},
    {"grid_i", (int32_t *)&str_PmuParameters.componentData.grid_i, NULL},
    {"grid_pf", (int32_t *)&str_PmuParameters.componentData.grid_pf, NULL},
    {"load_p", (int32_t *)&str_PmuParameters.componentData.load_p, NULL},
    {"load_v", (int32_t *)&str_PmuParameters.componentData.load_v, NULL},
    {"load_i", (int32_t *)&str_PmuParameters.componentData.load_i, NULL},
    {"load_pf", (int32_t *)&str_PmuParameters.componentData.load_pf, NULL},
    {"min_pf", (int32_t *)&str_PmuParameters.componentData.min_pf, NULL}
};
FileScop const size_t siz_NUM_PROD_KEYS = sizeof(arr_str_PmuProdData) / sizeof(arr_str_PmuProdData[0]);

FileScop const CCA_KeyToSchemaParamMap_t arr_str_PmuCompData[] = {
    #define X_PMU(key) {#key, (int32_t*)&str_PmuParameters.componentData.key, NULL},
    PMU_COMPONENT_DATA_XMACRO
    #undef X_PMU
};
FileScop const size_t siz_NUM_COMP_KEYS = sizeof(arr_str_PmuCompData) / sizeof(arr_str_PmuCompData[0]);

const CCA_KeyToSchemaCodeMap_t arr_str_PmuFaultCodes[] = {
    #define X_PMU(code) {#code, offsetof(SCH_PmuCodes_t, fault.code)},
    PMU_CODES_FAULT_XMACRO
    #undef X_PMU
};
FileScop const size_t siz_NUM_FAULT_KEYS = sizeof(arr_str_PmuFaultCodes) / sizeof(arr_str_PmuFaultCodes[0]);

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

ModScop void CCA_Pmu_Init(void)
{
    memset(&str_PmuParameters, 0, CCA_PMU_PARAMS_SIZE);
}

ModScop int CCA_Pmu_InitShadows(void)
{
    int int_rc = EXIT_SUCCESS;

    // Subscribe to PMU_Settings Shadow sub-topics
    int_rc = CCA_AWS_SubscribeToShadowSubtopics(CCA_sz_PMU_SHADOW_NAME, cca_sz_pmu_update_delta_topic, 
                                                cca_sz_pmu_update_accepted_topic, cca_sz_pmu_update_rejected_topic);
    // Publish PMU Settings
    if (int_rc == EXIT_SUCCESS)
    {
        ArduinoJson::StaticJsonDocument<CCA_PMU_u16_SETTINGS_JSON_DOC_SIZE> jd_PmuSettingsReported;
        ArduinoJson::JsonObject jo_Product = jd_PmuSettingsReported["state"]["reported"].createNestedObject("product");
        jo_Product["id"] = cca_arr_pmu_component_uuids[0];
        ArduinoJson::JsonObject jo_ProductDatum = jo_Product.createNestedObject("datum");
        
        char* sz_PmuResponseBuffer[siz_NUM_SETTINGS_KEYS] = {NULL};
        // Retrieve the current settings from the PMU
        int_rc = CCA_Icc_GetEMSConfiguration(CCA_sz_UNIT_TYPE_PMU, COD_u8_NULL_UNIT_ID, siz_NUM_SETTINGS_KEYS, 
                                                arr_sz_PmuSettings, sz_PmuResponseBuffer);
        if (int_rc != 0)
        {
            LOG_INF("Failed to getPMUData for %s, index: %u, with Error: %i",
                    CCA_sz_UNIT_TYPE_PMU, COD_u8_NULL_UNIT_ID, int_rc);
        }
        for (uint8_t i = 0; i < siz_NUM_SETTINGS_KEYS; i++)
        {
            char sz_TempValue[MAX_VALUE_SIZE];
            if (sz_PmuResponseBuffer[i] != NULL)
            {
                sprintf(sz_TempValue, "%s", sz_PmuResponseBuffer[i]);
            }
            else 
            {
                sprintf(sz_TempValue, "%s", "");
            }
            jo_ProductDatum[arr_sz_PmuSettings[i]] = sz_TempValue;
        }
        
        char sz_PmuSettingsReport[CCA_PMU_u16_SETTINGS_JSON_DOC_SIZE];
        ArduinoJson::serializeJson(jd_PmuSettingsReported, sz_PmuSettingsReport);
        LOG_INF("pmuSettingsReport: %s", sz_PmuSettingsReport);
        int_rc = CCA_AWS_PublishToTopic(&str_MqttContext, cca_sz_pmu_update_topic, strlen(cca_sz_pmu_update_topic),
                                        sz_PmuSettingsReport, strlen(sz_PmuSettingsReport));
    }
    else
    {
        LOG_ERR("Failed to initialize pmu_settings shdadow.\n");
    }
    return int_rc;
}

ModScop int CCA_Pmu_UpdateComponentData(uint8_t u8_UnitIndex)
{
    (void)(u8_UnitIndex); //Suppress unused parameter warning

    const char *sz_ProductUUID = cca_sz_thing_name;
    char sz_MqttPublishTopic[60];
    const uint8_t u8_PmuQty = CCA_Int_GetUnitQuantity(PMU);
    int int_rc = 0;
    bool bol_CommFailure = false;

    if (u8_PmuQty > 0)
    {
        ArduinoJson::StaticJsonDocument<CCA_u16_COMPONENT_JSON_DOC_SIZE> jd_PmuComponentData;
        sprintf(sz_MqttPublishTopic, "powersave/component/%s", cca_arr_pmu_component_uuids[0]);
        // Build PMU Component Data document
        jd_PmuComponentData["id"] = cca_arr_pmu_component_uuids[0];
        jd_PmuComponentData["product_id"] = sz_ProductUUID;
        jd_PmuComponentData["type"] = CCA_sz_UNIT_TYPE_PMU;
        jd_PmuComponentData["mfr"] = cca_sz_PMU_MFR;
        bol_CommFailure = false;
        uint32_t u32_PmuParametersSize = CCA_PMU_PARAMS_SIZE;
        int_rc = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_PMU, 0, &str_PmuParameters, &u32_PmuParametersSize);
        if (int_rc != 0 || u32_PmuParametersSize != CCA_PMU_PARAMS_SIZE)
        {
            LOG_INF("Failed to getEMSData for %s, index: %u, bytes: %d, with Error: %i",
                        CCA_sz_UNIT_TYPE_PMU, 0, u32_PmuParametersSize, int_rc);
            bol_CommFailure = true;
        }
        //Populate the JSON Document with the PMU Data, but only for the following keys:
        CCA_Int_AppendToJsonDoc(jd_PmuComponentData, arr_str_PmuCompData, siz_NUM_COMP_KEYS, bol_CommFailure);
        // Check for alarms, faults, and statuses
        ArduinoJson::JsonArray ja_Alarms = jd_PmuComponentData.createNestedArray("alarms");
        ArduinoJson::JsonArray ja_Faults = jd_PmuComponentData.createNestedArray("faults");
        ArduinoJson::JsonArray ja_Status = jd_PmuComponentData.createNestedArray("status");
        //There is no PMU alarm or status codes, only faults
        CCA_Int_AppendCodesToJsonArray(ja_Faults, arr_str_PmuFaultCodes, siz_NUM_FAULT_KEYS, &str_PmuParameters.codes);

        char sz_PmuComponentData[CCA_u16_COMPONENT_JSON_DOC_SIZE];
        ArduinoJson::serializeJson(jd_PmuComponentData, sz_PmuComponentData);
        // Validate the JSON prior to publish
        if (!jd_PmuComponentData.isNull())
        {
            CCA_AWS_PublishToTopic(&str_MqttContext, sz_MqttPublishTopic, strlen(sz_MqttPublishTopic),
                        sz_PmuComponentData, strlen(sz_PmuComponentData));
        }
        else
        {
            // Print the error messages
            LOG_ERR("ERROR: INVALID JSON during AWS_PublishComponentData()");
            LOG_ERR("%s\n", sz_PmuComponentData);
        }
    }
    
    return (int_rc < 0) ? int_rc : (bol_CommFailure) ? -EIO : 0;
}

ModScop int CCA_Pmu_GetProductData(ArduinoJson::JsonDocument& jd_ProductData, ArduinoJson::JsonArray& ja_Alarms,
                                    ArduinoJson::JsonArray& ja_Faults)
{
    (void)(ja_Alarms); //Suppress unused parameter warning

    uint32_t u32_PmuParametersSize = CCA_PMU_PARAMS_SIZE;
    const uint8_t u8_PmuQty = CCA_Int_GetUnitQuantity(PMU);
    bool bol_CommFailure = false;
    int int_rc = -ENXIO;
    if (u8_PmuQty > 0)
    {
        int_rc = CCA_Icc_GetBufferedData(CCA_sz_UNIT_TYPE_PMU, 0, &str_PmuParameters, &u32_PmuParametersSize);
    }
    if (int_rc != 0 || u32_PmuParametersSize != CCA_PMU_PARAMS_SIZE)
    {
        LOG_INF("Failed to getEMSData for %s, index: %u, bytes %d, with Error: %i", 
                    CCA_sz_UNIT_TYPE_PMU, 0, u32_PmuParametersSize, int_rc);
        bol_CommFailure = true;
    }
    //Populate the JSON Document with the PMU Data, but only for the following keys:
    CCA_Int_AppendToJsonDoc(jd_ProductData, arr_str_PmuProdData, siz_NUM_PROD_KEYS, bol_CommFailure);
    //Add the PMU Faults to the array
    if (!bol_CommFailure)
    {
        CCA_Int_AppendCodesToJsonArray(ja_Faults, arr_str_PmuFaultCodes, siz_NUM_FAULT_KEYS, &str_PmuParameters.codes);
    }
    return (int_rc < 0) ? int_rc : (bol_CommFailure) ? -EIO : 0;
}

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

//EOF
