/**********************************************************************************************************************
* @file            CCA_EMS.h
*
* @brief           This component contains the shadow and data handling for all devices in the EMS system.
*
* @author          Devan Hone
* @date            06/12/2024
* @copyright       2024 Lion Energy LLC.
*                  All rights reserved
*
*********************************************************************************************************************/
/**
* @addtogroup      CCA
*                  Cloud Communication Application Module 
* 
* @ref             CCA_EMS.h 
*                  
*********************************************************************************************************************/
#include "COD_Compiler_Defines.h"

#ifndef LION_CCA_EMS_H__
#define LION_CCA_EMS_H__

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

/**
 * @brief Initialize the EMS module. This should only be called once and before any other 
 *        functions in the module are called.
 */
ModScop void CCA_EMS_Init(void);

/**
 * @brief Initializes the EMS shadows and subscribes to the necessary topics.
 * 
 * @returns EXIT_SUCCESS or 0 upon success, standard POSIX error code upon failure.
 */
ModScop int CCA_EMS_InitShadows(void);

/**
 * @brief Triggers component data to be read and published to AWS for all components in the system.
 * 
 * @returns EXIT_SUCCESS or 0 upon success, standard POSIX error code upon failure.
 */
ModScop int CCA_EMS_UpdateComponentData(void);

/**
 * @brief Triggers product data to be read and published to AWS for the EMS product.
 * 
 * @returns EXIT_SUCCESS or 0 upon success, standard POSIX error code upon failure.
 */
ModScop int CCA_EMS_UpdateProductData(void);

#endif /* LION_CCA_EMS_H__ */
//EOF
