/**********************************************************************************************************************
* @file            CCA_Gw.h
*
* @brief           Handles all the data setting and retrieval for the GW device within the CCA module.
*
* @author          Devan Hone
* @date            06/11/2024
* @copyright       2024 Lion Energy LLC.
*                  All rights reserved
*
*********************************************************************************************************************/
/**
 * @addtogroup      CCA
 *                  Cloud Communication Application Module
 *
 * @ref             CCA_Gw.h
*                  
*********************************************************************************************************************/
#include "COD_Compiler_Defines.h"

#ifndef LION_CCA_GW_H__
#define LION_CCA_GW_H__

#include "CCA_Internal.h"

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

/**
 * @brief Initialize the GW module. This should only be called once and before any other 
 *        functions in the module are called.
 */
ModScop void CCA_Gw_Init(void);

/**
 * @brief Triggers component data to be read and published to AWS for each GW component in the system.
 * 
 * @param u8_UnitIndex Index of GW unit data to update. Set to @ref COD_u8_NULL_UNIT_ID to update all units data.
 * @returns EXIT_SUCCESS or 0 upon success, standard POSIX error code upon failure.
 */
ModScop int CCA_Gw_UpdateComponentData(uint8_t u8_UnitIndex);

/**
 * @brief Triggers product data to be read and populated in the JSON document provided.
 * 
 * @param jd_ProductData JSON document to store the product data in.
 * @param ja_Alarms JSON array to store the alarms in.
 * @param ja_Faults JSON array to store the faults in.
 * @returns EXIT_SUCCESS or 0 upon success, standard POSIX error code upon failure.
 */
ModScop int CCA_Gw_GetProductData(ArduinoJson::JsonDocument& jd_ProductData, ArduinoJson::JsonArray& ja_Alarms,
                                    ArduinoJson::JsonArray& ja_Faults);

#endif // LION_CCA_GW_H__
//EOF
