/**********************************************************************************************************************
* @file            CCA_ReqHandler.h
*
* @brief           These files contain the handlers for AWS command requests.
*
* @author          Devan Hone
* @date            06/13/2024
* @copyright       2024 Lion Energy LLC.
*                  All rights reserved
*
*********************************************************************************************************************/
/**
* @addtogroup      CCA
*                  Cloud Communication Application Module  
* 
* @ref             CCA_ReqHandler.h 
*                  
*********************************************************************************************************************/
#include "COD_Compiler_Defines.h"

#ifndef LION_CCA_REQHANDLER_H__
#define LION_CCA_REQHANDLER_H__

#include "CCA_Internal.h"

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/
ModScop typedef void (*cca_HandlerFunction_t)(ArduinoJson::JsonDocument&,ArduinoJson::JsonDocument&);

/**
 * @brief   Evaluates if the requested function is supported based on the ID. 
 * @param   sz_RequestID    Requested ID to be validated
 * 
 * @returns EXIT_SUCESS or standard POSIX error code upon failure.
*/
ModScop int CCA_Rqh_ValidateRequest(const char *sz_RequestID, cca_HandlerFunction_t *ptr_HandlerFunction);


#endif // LION_CCA_REQHANDLER_H__
//EOF
