/**********************************************************************************************************************
* @file            CCA_Version.h
*
* @brief           Handles version shadow updates and manages versioning information for all components 
                   within the system. This includes the CCU, DCU, and PMU
*
* @author          Devan Hone
* @date            06/28/2024
* @copyright       2024 Lion Energy LLC.
*                  All rights reserved
*
*********************************************************************************************************************/
/**
 * @addtogroup      CCA
 *                  Cloud Communication Application Module
 *
 * @ref             CCA_Version.h
*                  
*********************************************************************************************************************/
#include "COD_Compiler_Defines.h"

#ifndef LION_CCA_VERSION_H__
#define LION_CCA_VERSION_H__

#include "CCA_Internal.h"

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

/**
 * @brief Initialize the Version module. This should only be called once and before any other 
 *        functions in the module are called.
 */
ModScop void CCA_Version_Init(void);

/**
 * @brief Initializes the Version shadows and subscribes to the necessary topics.
 * 
 * @returns EXIT_SUCCESS or 0 upon success, standard POSIX error code upon failure.
 */
ModScop int CCA_Version_InitShadows(void);

/**
 * @brief Updates the version information JSON document with the provided JSON object.
 * 
 * @details If the object already exists, it will be updated. If it does not exist, it will be created.
 * 
 * @param jo_Obj JSON object to update the version information with.
 * 
 * @return EXIT_SUCCESS or 0 upon success, standard posix errors upon failure.
 */
ModScop int CCA_Version_UpdateVersionInfoWith(ArduinoJson::JsonObject& jo_Obj);

/**
 * @brief Publishes the version info to the AWS cloud.
 * 
 * @returns EXIT_SUCCESS or 0 upon success, standard POSIX error code upon failure.
 */
ModScop int CCA_Version_PublishVersionUpdate(void);


#endif // LION_CCA_VERSION_H__
//EOF
