#ifndef LION_RTOS_CONDVAR_H__
#define LION_RTOS_CONDVAR_H__

/* ======================================================================= */
/* =========================== I N C L U D E S =========================== */
/* ======================================================================= */

#include "RTOS_Timeout.h"
#include "RTOS_Mutex.h"

#ifdef __cplusplus
extern "C"
{
#endif

    /* ======================================================================= */
    /* ============================ T Y P E D E F S ========================== */
    /* ======================================================================= */

#ifdef __ZEPHYR__

    #include <zephyr/kernel.h>

    typedef struct k_condvar RTOS_CondVar_t;

    static inline int RTOS_CondVarInit(RTOS_CondVar_t *ptrCondVar) { return k_condvar_init(ptrCondVar); }

    static inline int RTOS_CondVarSignal(RTOS_CondVar_t *ptrCondVar) { return k_condvar_signal(ptrCondVar); }

    static inline int RTOS_CondVarBroadcast(RTOS_CondVar_t *ptrCondVar) { return k_condvar_broadcast(ptrCondVar); }

    static inline int RTOS_CondVarWait(RTOS_CondVar_t *ptrCondVar, RTOS_Mutex_t *ptrMutex, RTOS_Timeout_t strTimeout)
    {
        return k_condvar_wait(ptrCondVar, ptrMutex, strTimeout);
    }

#elif __unix__

    #include <errno.h>
    #include <pthread.h>

typedef pthread_cond_t RTOS_CondVar_t;

static inline int RTOS_CondVarInit(RTOS_CondVar_t *ptrCondVar)
{
    return -pthread_cond_init(ptrCondVar, NULL);
}

static inline int RTOS_CondVarSignal(RTOS_CondVar_t *ptrCondVar)
{
    return -pthread_cond_signal(ptrCondVar);
}

static inline int RTOS_CondVarBroadcast(RTOS_CondVar_t *ptrCondVar)
{
    return -pthread_cond_broadcast(ptrCondVar);
}

static inline int RTOS_CondVarWait(RTOS_CondVar_t *ptrCondVar, RTOS_Mutex_t *ptrMutex, RTOS_Timeout_t strTimeout)
{
    int intRC;
    if (RTOS_TIMEOUT_EQ(strTimeout, RTOS_NO_WAIT))
    {
        /* Unsure if this is proper behavior, but it doesn't make sense to support NO_WAIT on a condvar? */
        intRC = EAGAIN;
    }
    else if (RTOS_TIMEOUT_EQ(strTimeout, RTOS_FOREVER))
    {
        intRC = pthread_cond_wait(ptrCondVar, ptrMutex);
    }
    else
    {
        struct timespec strTime = RTOS_TimeoutToTimespec(strTimeout);
        intRC                   = pthread_cond_clockwait(ptrCondVar, ptrMutex, CLOCK_MONOTONIC, &strTime);
    }
    /* Discrepancy: Linux pthread returns ETIMEDOUT, Zephyr returns EAGAIN? */
    return -intRC;
}

#endif

    /* ======================================================================= */
    /* ============================= D E F I N E S =========================== */
    /* ======================================================================= */

    /* ======================================================================= */
    /* ========================== P R O T O T Y P E S ======================== */
    /* ======================================================================= */

#ifdef __cplusplus
}
#endif

#endif /* LION_RTOS_CONDVAR_H__ */