#ifndef LION_RTOS_MUTEX_H__
#define LION_RTOS_MUTEX_H__

/* ======================================================================= */
/* =========================== I N C L U D E S =========================== */
/* ======================================================================= */

#include "RTOS_Timeout.h"

#ifdef __cplusplus
extern "C"
{
#endif

    /* ======================================================================= */
    /* ============================ T Y P E D E F S ========================== */
    /* ======================================================================= */

#ifdef __ZEPHYR__

    #include <zephyr/kernel.h>

    typedef struct k_mutex RTOS_Mutex_t;

    static inline int RTOS_MutexInit(RTOS_Mutex_t *ptrMutex) { return k_mutex_init(ptrMutex); }

    static inline void RTOS_MutexUnlock(RTOS_Mutex_t *ptrMutex) { k_mutex_unlock(ptrMutex); }

    static inline int RTOS_MutexLock(RTOS_Mutex_t *ptrMutex, RTOS_Timeout_t strTimeout)
    {
        return k_mutex_lock(ptrMutex, strTimeout);
    }

#elif __unix__

    #include <errno.h>
    #include <pthread.h>

typedef pthread_mutex_t RTOS_Mutex_t;

static inline int RTOS_MutexInit(RTOS_Mutex_t *ptrMutex)
{
    pthread_mutexattr_t str_MutexAttr;
    pthread_mutexattr_init(&str_MutexAttr);
    pthread_mutexattr_settype(&str_MutexAttr, PTHREAD_MUTEX_RECURSIVE);
    return -pthread_mutex_init(ptrMutex, &str_MutexAttr);
}

static inline void RTOS_MutexUnlock(RTOS_Mutex_t *ptrMutex) { pthread_mutex_unlock(ptrMutex); }

static inline int RTOS_MutexLock(RTOS_Mutex_t *ptrMutex, RTOS_Timeout_t strTimeout)
{
    int intRC;
    if (RTOS_TIMEOUT_EQ(strTimeout, RTOS_NO_WAIT))
    {
        intRC = pthread_mutex_trylock(ptrMutex);
    }
    else if (RTOS_TIMEOUT_EQ(strTimeout, RTOS_FOREVER))
    {
        intRC = pthread_mutex_lock(ptrMutex);
    }
    else
    {
        struct timespec strTime = RTOS_TimeoutToTimespec(strTimeout);
        intRC                   = pthread_mutex_clocklock(ptrMutex, CLOCK_MONOTONIC, &strTime);
    }
    return -intRC;
}

#endif

    /* ======================================================================= */
    /* ============================= D E F I N E S =========================== */
    /* ======================================================================= */

    /* ======================================================================= */
    /* ========================== P R O T O T Y P E S ======================== */
    /* ======================================================================= */

#ifdef __cplusplus
}
#endif

#endif /* LION_RTOS_MUTEX_H__ */