/**********************************************************************************************************************
* @file            IAL_Control.h
*
* @brief           Provides APIs for control related ICC functions and events for the CCU.
*                  
*
* @author          Devan Hone
* @date            07/25/2024
* @copyright       2024 Lion Energy LLC.
*                  All rights reserved
*
*********************************************************************************************************************/
/**
* @addtogroup      IAL
*                  Identifier Description for the ICC Abstraction Library (IAL) Module  
* 
* @ref             IAL.h 
*                  
*********************************************************************************************************************/
#include "COD_Compiler_Defines.h"

#ifndef LION_IAL_CONTROL_H__
#define LION_IAL_CONTROL_H__

/*********************************************************************************************************************
* @cond                                              GLOBAL SCOPE                                             @endcond
*********************************************************************************************************************/

#include <lion/ial/IAL.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**
 * @brief  Send a restart request to the two boards of the EMS using the Inter-Controller Communication (ICC) Service. 
 * 
 * @param sz_UnitType dcu (Domain Controller Unit) or pmu (Power Metering Unit)
 * 
 * @returns EXIT_SUCCESS or 0 upon success, EXIT_FAILURE upon failure
 */
GlobScop int IAL_Ctl_RestartEMS(const char *sz_UnitType);

/**
 * @brief  Send a request to reinitialize communication services to the DCU (Domain Controller Unit) using the 
 *         Inter-Controller Communication (ICC) Service. 
 * 
 * @returns EXIT_SUCCESS or 0 upon success, EXIT_FAILURE upon failure
 */
GlobScop int IAL_Ctl_ReinitComDCU(void);

/**
 * @brief  Send a request to all units in the system to publish their current version information.
 * 
 * @returns EXIT_SUCCESS or 0 upon success, standard posix errors upon failure
 */
GlobScop int IAL_Ctl_RequestVersionInfo(void);

#ifdef __cplusplus
}
#endif

#endif /* LION_IAL_CONTROL_H__ */
//EOF
