#include <lion/ial/ccu/IAL_Control.h>
#include <lion/log/log.h>
#include <string.h>
#include <zcbor_decode.h>
#include <zcbor_encode.h>

LOG_MODULE_REGISTER(ial_control, LOG_LEVEL_INF);

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

/*********************************************************************************************************************
* @cond                                              MODULE SCOPE                                             @endcond
*********************************************************************************************************************/

ModScop extern ICC_Context_t *ial_ptr_IccCtx; //Defined in IAL.c

/*********************************************************************************************************************
* @cond                                              GLOBAL SCOPE                                             @endcond
*********************************************************************************************************************/

GlobScop int IAL_Ctl_RestartEMS(const char *sz_UnitType)
{
    // Build the Request
    uint8_t u8_RequestBuffer[] = { 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05 };

    // Set the ICC Function ID based on unit type
    uint32_t u32_FunctionID;
    if (strcmp(sz_UnitType, IAL_sz_UNIT_TYPE_DCU) == EXIT_SUCCESS)
    {
        u32_FunctionID = IAL_DCU_RESTART_ID;
    }
    else if (strcmp(sz_UnitType, IAL_sz_UNIT_TYPE_PMU) == EXIT_SUCCESS) 
    {
        u32_FunctionID = IAL_PMU_RESTART_ID;
    }
    else {
        LOG_INF("IAL_Ctl_RestartEMS Error - Invalid unitType: \"%s\"", sz_UnitType);
        return -EINVAL;
    }

    // Send the Request
    uint32_t u32_RequestLength = sizeof(u8_RequestBuffer);
    FileScop uint8_t arr_s32_EncodedResponses[IAL_u16_ICC_RESPONSE_BUFFER_SIZE] = {0};
    uint32_t u32_ResponseLength = sizeof(arr_s32_EncodedResponses);
    int int_Rc = ICC_Function_Call(ial_ptr_IccCtx, u32_FunctionID, u8_RequestBuffer, u32_RequestLength, 
                                    arr_s32_EncodedResponses, &u32_ResponseLength, RTOS_SECONDS(IAL_u8_ICC_CALL_TIMEOUT_S));
    
    if (int_Rc < 0) {
        LOG_INF("ICC_Function_Call Error %d \"%s\"", int_Rc, strerror(-int_Rc));
        return int_Rc;
    }
    return int_Rc;
}

GlobScop int IAL_Ctl_ReinitComDCU(void)
{
     // Build the Request
    uint8_t u8_RequestBuffer[] = { 0x00 };  // There is no body to the request

    // Send the Request
    uint32_t u32_RequestLength = 1;
    FileScop uint8_t u8_EncodedResponses[IAL_u16_ICC_RESPONSE_BUFFER_SIZE] = {0};
    uint32_t u32_ResponseLength = sizeof(u8_EncodedResponses);
    int int_Rc = ICC_Function_Call(ial_ptr_IccCtx, IAL_DCU_REINIT_COM_ID, u8_RequestBuffer, u32_RequestLength, 
                                    u8_EncodedResponses, &u32_ResponseLength, RTOS_SECONDS(IAL_u8_ICC_CALL_TIMEOUT_S));
    if (int_Rc < 0) {
        LOG_INF("ICC_Function_Call Error %d \"%s\"", int_Rc, strerror(-int_Rc));
        return int_Rc;
    }

    // Decode the response
    int32_t arr_s32_Statuses[1];
    ZCBOR_STATE_D(CBORDecodeState, 1, u8_EncodedResponses, u32_ResponseLength, 3);
    if (zcbor_list_start_decode(CBORDecodeState) == 0)
    {
        LOG_INF("Decoding CBOR Error");
    }
    else 
    {
        zcbor_int32_decode(CBORDecodeState, &arr_s32_Statuses[0]);
    }
    zcbor_list_end_decode(CBORDecodeState);
    if (arr_s32_Statuses[0] != EXIT_SUCCESS)
    {
        LOG_INF("Reinitialize DCU Communications Error %d \"%s\"", arr_s32_Statuses[0], strerror(-arr_s32_Statuses[0]));
    }
    return arr_s32_Statuses[0];
}

GlobScop int IAL_Ctl_RequestVersionInfo(void)
{
    return ICC_Event_Dispatch(ial_ptr_IccCtx, IAL_VERSION_INFO_ID, NULL, 0, RTOS_SECONDS(IAL_u8_ICC_EVENT_TIMEOUT_S));
}

/*********************************************************************************************************************
* @cond                                               FILE SCOPE                                              @endcond
*********************************************************************************************************************/

//EOF
